/////////////////////////////////////////
////
/// Fichero: ejemplo_excepcion_2.cpp
///
//////////////////////////////////////////

#include <iostream>
#include <vector>

// Necesario para rand() y srand()
#include <cstdlib>    

//Necesario para derivar de la clase exception
#include <exception>  

using namespace std;

/////////////////////////////////////////////////////////////////////////
///
/// Clase: Excepcion_No_Encontrado    
///
/// Descripcion: representa una clase de excepcion 
///              correspondiente a un objeto no encontrado
/// Representacion: no possee atributos
///
///
/// Herencia: hereda de la clase predefinida "exception" 
///
/////////////////////////////////////////////////////////////////////////

class Excepcion_No_Encontrado : public exception 
{
  public:

    /// Constructor vacío
    /// Usa un iniciador para invocar al construtor de la clase base
    Excepcion_No_Encontrado():exception() 
    {
    };

    /// Funcion de consulta: indica el tipo de error
    const char* mensaje_error()
     {
        return ("Excepcion: Elemento No Encontrado en el Vector");
      }
};




/////////////////////////////////////////////////////////////////////////
/// Funcion: busqueda
///
/// Objetivo: buscar una elemento en un vector de enteros 
/// Descripcion: 
///     Recorre el vector para comprobar
///     si alguna de las componentes coincide con el elemento buscado
/// Entrada:       
///   - v: vector de enteros
///   - x: dato buscado 
/// Devuelve: 
///     El indice de la componente en la que se encuentra el objeto buscado    
///     En caso contrario, lanza una excepcion de objeto no encontrado
///  
/// Dependencias: 
///////////////////////////////////////////////////////////////////////// 

int busqueda(vector<int> v, int x)
{
  unsigned int i;
  bool encontrado = false;

  ///
  for (i = 0; i < v.size();i++) 
    if (v[i] == x) 
    {
      encontrado = true;
      break;
    }
    
  ///
  if (encontrado) 
    return i;
  else 
    //Lanza la excepcion usando la clase Excepcion_No_Encontrado
    throw Excepcion_No_Encontrado(); 

  return (0);
}



/////////////////////////////////////////////////////////////////

int main(void)
{
  /// Se define un vector de enteros utilizando la biblioteca estandar de plantillas
  vector <int> v(10);

  int i, j;

  cout << " Ejemplo de Excepciones en C++ usando la clase << exception>> \n";


  // Usa los segundos del sistema para establecer la semilla
  srand(time(NULL)); 

  /// Asigna valores aleatorios a la componentes del vector
  for (unsigned int i = 0;i < v.size(); i++)
    v[i] = (1 + rand() % 10);
 
  /// Muestra las componentes del vector
  for (unsigned int i = 0; i < v.size(); i++) 
    cout << v[i] << " ";

  cout << endl;

 do {
  /// Se solicita el dato buscado
  cout << "\n Introduce el elemento a buscar (-1 para terminar) ";
  cin >> i;

  /// Se intenta localizar el dato buscado
  try 
  { 
    cout << "\n\t Estoy dentro del bloque try \n";

    j = busqueda(v, i);

    cout << "\n\t Estoy despues del throw: no se ha lanzado la excepcion \n";

    cout << "\n\t El elemento " << v[j] << " esta en la posicion "<< j << endl;
  
  }

  /// Manejador de excepciones
  catch (Excepcion_No_Encontrado &ex) 
  { /// Toma la referencia del objeto
    /// No es necesario coger la copia y se ahorra memoria.

     cout << "\n\t Error capturado ---> " << ex.mensaje_error() << endl;
  }

 } while ( i != -1);

  cout << "Fin \n";
  
  return (0);
}

